using System;
using System.Xml;

namespace ExpressionTree
{

	// TNode represents a simple arithmetic term of type
	// term operator term
	// operator: one of  +  -  *  /  #
	//		# is used as dummy opcode if the term is a constant
	public class TNode
	{
		public const char terminateOpCode = '#';

		char opCode = terminateOpCode;
		double result;
		TNode opA;
		TNode opB;

		public double Result
		{
			get { return result; }
			set { result = value; }
		}

		public char OpCode
		{
			get { return opCode; }
			set { opCode = value; }
		}

		public TNode OpA 
		{
			get { return opA; }
			set { opA = value; }
		}

		public TNode OpB 
		{
			get { return opB; }
			set { opB = value; }
		}

		public TNode()
		{
		}

		public TNode(char opCode)
		{
			this.opCode = opCode;
		}

		public TNode(double result )
		{
			this.result = result;
		}

		public override string ToString()
		{
			string text = "";
			if (opCode != terminateOpCode)
				text += String.Format("[{0}] ", opCode);
			text += String.Format("{0:f1}", result);
			return text;
		}

		public void AddExpression(double a, double b, char opCode)
		{
			opA = new TNode(a);
			opB = new TNode(b);
			this.opCode = opCode;
		}

		public void AddExpression(char opCode)
		{
			opA = new TNode(0.0);
			opB = new TNode(0.0);
			this.opCode = opCode;
		}

		public double Calculate()
		{
			if (opA == null || opB == null)
				return result;

			switch (opCode)
			{
				case '+':
					result = opA.Calculate() + opB.Calculate();
					break;
				case '-':
					result = opA.Calculate() - opB.Calculate();
					break;
				case '*':
					result = opA.Calculate() * opB.Calculate();
					break;
				case '/':
					result = opA.Calculate() / opB.Calculate();
					break;
				case terminateOpCode:	// leaf
				default:
					break;
			}
			return result;
		}

		public void WriteAsXMLNode (XmlTextWriter xr)
		{
			xr.WriteStartElement("Term");
			if (opCode != terminateOpCode)
			{
				xr.WriteAttributeString("OpCode", XmlConvert.ToString(opCode));
			}
			xr.WriteElementString("Value", XmlConvert.ToString(result));
			if (opA != null)
				opA.WriteAsXMLNode(xr);
			if (opB != null)
				opB.WriteAsXMLNode(xr);
			xr.WriteEndElement();
		}

	}
}
